@@ -60,14 +60,14 @@
     AC_CHECK_TYPES([ptrdiff_t])      defines HAVE_PTRDIFF_T
     AC_CHECK_SIZEOF([unsigned long]) defines SIZEOF_UNSIGNED_LONG
     AC_CHECK_SIZEOF([void *])        defines SIZEOF_VOID_P
-                                             HAVE_LANGINFO_THOUSANDS_SEP
+                                             HAVE_LANGINFO_THOUSEP
 
     Note that the file estream.m4 provides the autoconf macro
     ESTREAM_PRINTF_INIT which runs all required checks.
     See estream-printf.h for ways to tune this code.
 
   Missing stuff:  wchar and wint_t
-                  thousands_sep in pr_float.
+                  thousep in pr_float.
 
 */
 
@@ -77,9 +77,6 @@
 
 #if defined(_WIN32) && !defined(HAVE_W32_SYSTEM)
 # define HAVE_W32_SYSTEM 1
-# if defined(__MINGW32CE__) && !defined (HAVE_W32CE_SYSTEM)
-#  define HAVE_W32CE_SYSTEM
-# endif
 #endif
 
 #include <stdio.h>
@@ -89,18 +86,14 @@
 #include <stdarg.h>
 #include <errno.h>
 #include <stddef.h>
-#include <assert.h>
 #if defined(HAVE_INTMAX_T) || defined(HAVE_UINTMAX_T)
 # ifdef HAVE_STDINT_H
 #  include <stdint.h>
 # endif
 #endif
-#ifdef HAVE_LANGINFO_THOUSANDS_SEP
+#ifdef HAVE_LANGINFO_THOUSEP
 #include <langinfo.h>
 #endif
-#ifdef HAVE_W32CE_SYSTEM
-#include <gpg-error.h>  /* ERRNO replacement.  */
-#endif
 #ifdef _ESTREAM_PRINTF_EXTRA_INCLUDE
 # include _ESTREAM_PRINTF_EXTRA_INCLUDE
 #endif
@@ -117,11 +110,7 @@
 #endif
 
 /* A wrapper to set ERRNO.  */
-#ifdef HAVE_W32CE_SYSTEM
-# define _set_errno(a)  gpg_err_set_errno ((a))
-#else
-# define _set_errno(a)  do { errno = (a); } while (0)
-#endif
+#define _set_errno(a)  do { errno = (a); } while (0)
 
 
 /* Calculate array dimension.  */
@@ -936,8 +925,8 @@ pr_integer (estream_printf_out_t outfnc, void *outfncarg,
     {
       int grouping = -1;
       const char * grouping_string =
-#ifdef HAVE_LANGINFO_THOUSANDS_SEP
-        nl_langinfo(THOUSANDS_SEP);
+#ifdef HAVE_LANGINFO_THOUSEP
+        nl_langinfo(THOUSEP);
 #else
         "'";
 #endif
@@ -1196,7 +1185,8 @@ pr_char (estream_printf_out_t outfnc, void *outfncarg,
 /* "s" formatting.  */
 static int
 pr_string (estream_printf_out_t outfnc, void *outfncarg,
-            argspec_t arg, value_t value, size_t *nbytes)
+           argspec_t arg, value_t value, size_t *nbytes,
+           gpgrt_string_filter_t sf, void *sfvalue, int string_no)
 {
   int rc;
   size_t n;
@@ -1204,7 +1194,11 @@ pr_string (estream_printf_out_t outfnc, void *outfncarg,
 
   if (arg->vt != VALTYPE_STRING)
     return -1;
-  string = value.a_string;
+  if (sf)
+    string = sf (value.a_string, string_no, sfvalue);
+  else
+    string = value.a_string;
+
   if (!string)
     string = "(null)";
   if (arg->precision >= 0)
@@ -1222,12 +1216,12 @@ pr_string (estream_printf_out_t outfnc, void *outfncarg,
     {
       rc = pad_out (outfnc, outfncarg, ' ', arg->width - n, nbytes);
       if (rc)
-        return rc;
+        goto leave;
     }
 
   rc = outfnc (outfncarg, string, n);
   if (rc)
-    return rc;
+    goto leave;
   *nbytes += n;
 
   if ((arg->flags & FLAG_LEFT_JUST)
@@ -1235,10 +1229,16 @@ pr_string (estream_printf_out_t outfnc, void *outfncarg,
     {
       rc = pad_out (outfnc, outfncarg, ' ', arg->width - n, nbytes);
       if (rc)
-        return rc;
+        goto leave;
     }
 
-  return 0;
+  rc = 0;
+
+ leave:
+  if (sf) /* Tell the filter to release resources.  */
+    sf (value.a_string, -1, sfvalue);
+
+  return rc;
 }
 
 
@@ -1337,14 +1337,18 @@ pr_bytes_so_far (estream_printf_out_t outfnc, void *outfncarg,
 
 
 /* Run the actual formatting.  OUTFNC and OUTFNCARG are the output
-   functions.  FORMAT is format string ARGSPECS is the parsed format
-   string, ARGSPECS_LEN the number of items in ARGSPECS.  VALUETABLE
-   holds the values and may be directly addressed using the position
-   arguments given by ARGSPECS.  MYERRNO is used for the "%m"
-   conversion. NBYTES well be updated to reflect the number of bytes
-   send to the output function. */
+ * functions.  FORMAT is format string ARGSPECS is the parsed format
+ * string, ARGSPECS_LEN the number of items in ARGSPECS.
+ * STRING_FILTER is an optional function to filter string (%s) args;
+ * it is called with the original string and the count of already
+ * processed %s arguments.  Its return value will be used instead of
+ * the original string.  VALUETABLE holds the values and may be
+ * directly addressed using the position arguments given by ARGSPECS.
+ * MYERRNO is used for the "%m" conversion. NBYTES well be updated to
+ * reflect the number of bytes send to the output function. */
 static int
 do_format (estream_printf_out_t outfnc, void *outfncarg,
+           gpgrt_string_filter_t sf, void *sfvalue,
            const char *format, argspec_t argspecs, size_t argspecs_len,
            valueitem_t valuetable, int myerrno, size_t *nbytes)
 {
@@ -1354,6 +1358,7 @@ do_format (estream_printf_out_t outfnc, void *outfncarg,
   int argidx = 0; /* Only used for assertion.  */
   size_t n;
   value_t value;
+  int string_no = 0;  /* Number of processed "%s" args.  */
 
   s = format;
   while ( *s )
@@ -1388,13 +1393,13 @@ do_format (estream_printf_out_t outfnc, void *outfncarg,
       s += arg->length;
       format = s;
 
-      assert (argidx < argspecs_len);
+      gpgrt_assert (argidx < argspecs_len);
       argidx++;
 
       /* Apply indirect field width and precision values.  */
       if (arg->width == STAR_FIELD_VALUE)
         {
-          assert (valuetable[arg->width_pos-1].vt == VALTYPE_INT);
+          gpgrt_assert (valuetable[arg->width_pos-1].vt == VALTYPE_INT);
           arg->width = valuetable[arg->width_pos-1].value.a_int;
           if (arg->width < 0)
             {
@@ -1404,7 +1409,7 @@ do_format (estream_printf_out_t outfnc, void *outfncarg,
         }
       if (arg->precision == STAR_FIELD_VALUE)
         {
-          assert (valuetable[arg->precision_pos-1].vt == VALTYPE_INT);
+          gpgrt_assert (valuetable[arg->precision_pos-1].vt == VALTYPE_INT);
           arg->precision = valuetable[arg->precision_pos-1].value.a_int;
           if (arg->precision < 0)
             arg->precision = NO_FIELD_VALUE;
@@ -1414,13 +1419,13 @@ do_format (estream_printf_out_t outfnc, void *outfncarg,
         value.a_string = strerror (myerrno);
       else
         {
-          assert (arg->vt == valuetable[arg->arg_pos-1].vt);
+          gpgrt_assert (arg->vt == valuetable[arg->arg_pos-1].vt);
           value = valuetable[arg->arg_pos-1].value;
         }
 
       switch (arg->conspec)
         {
-        case CONSPEC_UNKNOWN: assert (!"bug"); break;
+        case CONSPEC_UNKNOWN: gpgrt_assert (!"bug"); break;
 
         case CONSPEC_DECIMAL:
         case CONSPEC_UNSIGNED:
@@ -1443,8 +1448,12 @@ do_format (estream_printf_out_t outfnc, void *outfncarg,
           rc = pr_char (outfnc, outfncarg, arg, value, nbytes);
           break;
         case CONSPEC_STRING:
+          rc = pr_string (outfnc, outfncarg, arg, value, nbytes,
+                          sf, sfvalue, string_no++);
+          break;
         case CONSPEC_STRERROR:
-          rc = pr_string (outfnc, outfncarg, arg, value, nbytes);
+          rc = pr_string (outfnc, outfncarg, arg, value, nbytes,
+                          NULL, NULL, 0);
           break;
         case CONSPEC_POINTER:
           rc = pr_pointer (outfnc, outfncarg, arg, value, nbytes);
@@ -1478,6 +1487,7 @@ do_format (estream_printf_out_t outfnc, void *outfncarg,
 int
 _gpgrt_estream_format (estream_printf_out_t outfnc,
                        void *outfncarg,
+                       gpgrt_string_filter_t sf, void *sfvalue,
                        const char *format, va_list vaargs)
 {
   /* Buffer to hold the argspecs and a pointer to it.*/
@@ -1489,10 +1499,10 @@ _gpgrt_estream_format (estream_printf_out_t outfnc,
   struct valueitem_s valuetable_buffer[DEFAULT_MAX_VALUES];
   valueitem_t valuetable = valuetable_buffer;
 
-  int rc;     /* Return code. */
+  int rc;        /* Return code. */
   size_t argidx; /* Used to index the argspecs array.  */
   size_t validx; /* Used to index the valuetable.  */
-  int max_pos;/* Highest argument position.  */
+  int max_pos;   /* Highest argument position.  */
 
   size_t nbytes = 0; /* Keep track of the number of bytes passed to
                         the output function.  */
@@ -1565,7 +1575,11 @@ _gpgrt_estream_format (estream_printf_out_t outfnc,
   else
     {
       for (validx=0; validx < DIM(valuetable_buffer); validx++)
-        valuetable[validx].vt = VALTYPE_UNSUPPORTED;
+        {
+          valuetable[validx].vt = VALTYPE_UNSUPPORTED;
+          memset (&valuetable[validx].value, 0,
+                  sizeof valuetable[validx].value);
+        }
     }
   for (argidx=0; argidx < argspecs_len; argidx++)
     {
@@ -1602,7 +1616,7 @@ _gpgrt_estream_format (estream_printf_out_t outfnc,
 /*     fprintf (stderr, "%2d: vt=%d\n", validx, valuetable[validx].vt); */
 
   /* Everything has been collected, go ahead with the formatting.  */
-  rc = do_format (outfnc, outfncarg, format,
+  rc = do_format (outfnc, outfncarg, sf, sfvalue, format,
                   argspecs, argspecs_len, valuetable, myerrno, &nbytes);
 
   goto leave;
@@ -1642,7 +1656,8 @@ _gpgrt_estream_printf (const char *format, ...)
   va_list arg_ptr;
 
   va_start (arg_ptr, format);
-  rc = _gpgrt_estream_format (plain_stdio_out, stderr, format, arg_ptr);
+  rc = _gpgrt_estream_format (plain_stdio_out, stderr, NULL, NULL,
+                              format, arg_ptr);
   va_end (arg_ptr);
 
   return rc;
@@ -1656,7 +1671,8 @@ _gpgrt_estream_fprintf (FILE *fp, const char *format, ...)
   va_list arg_ptr;
 
   va_start (arg_ptr, format);
-  rc = _gpgrt_estream_format (plain_stdio_out, fp, format, arg_ptr);
+  rc = _gpgrt_estream_format (plain_stdio_out, fp, NULL, NULL,
+                              format, arg_ptr);
   va_end (arg_ptr);
 
   return rc;
@@ -1666,7 +1682,8 @@ _gpgrt_estream_fprintf (FILE *fp, const char *format, ...)
 int
 _gpgrt_estream_vfprintf (FILE *fp, const char *format, va_list arg_ptr)
 {
-  return _gpgrt_estream_format (plain_stdio_out, fp, format, arg_ptr);
+  return _gpgrt_estream_format (plain_stdio_out, fp, NULL, NULL,
+                                format, arg_ptr);
 }
 
 
@@ -1721,7 +1738,8 @@ _gpgrt_estream_vsnprintf (char *buf, size_t bufsize,
   parm.count = 0;
   parm.used = 0;
   parm.buffer = bufsize?buf:NULL;
-  rc = _gpgrt_estream_format (fixed_buffer_out, &parm, format, arg_ptr);
+  rc = _gpgrt_estream_format (fixed_buffer_out, &parm, NULL, NULL,
+                              format, arg_ptr);
   if (!rc)
     rc = fixed_buffer_out (&parm, "", 1); /* Print terminating Nul.  */
   if (rc == -1)
@@ -1816,7 +1834,8 @@ _gpgrt_estream_vasprintf (char **bufp, const char *format, va_list arg_ptr)
       return -1;
     }
 
-  rc = _gpgrt_estream_format (dynamic_buffer_out, &parm, format, arg_ptr);
+  rc = _gpgrt_estream_format (dynamic_buffer_out, &parm, NULL, NULL,
+                              format, arg_ptr);
   if (!rc)
     rc = dynamic_buffer_out (&parm, "", 1); /* Print terminating Nul.  */
   /* Fixme: Should we shrink the resulting buffer?  */
@@ -1833,7 +1852,7 @@ _gpgrt_estream_vasprintf (char **bufp, const char *format, va_list arg_ptr)
       *bufp = NULL;
       return -1;
     }
-  assert (parm.used);   /* We have at least the terminating Nul.  */
+  gpgrt_assert (parm.used);   /* We have at least the terminating Nul.  */
   *bufp = parm.buffer;
   return parm.used - 1; /* Do not include that Nul. */
 }
