@@ -1,22 +1,23 @@
 /* gpgme.c - GnuPG Made Easy.
-   Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2012,
-                 2014 g10 Code GmbH
-
-   This file is part of GPGME.
-
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * Copyright (C) 2000 Werner Koch (dd9jn)
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2012,
+ *               2014, 2015 g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #if HAVE_CONFIG_H
@@ -38,6 +39,7 @@
 #include "debug.h"
 #include "priv-io.h"
 #include "sys-util.h"
+#include "mbox-util.h"
 
 
 /* The default locale.  */
@@ -71,10 +73,19 @@ gpgme_set_global_flag (const char *name, const char *value)
       _gpgme_dirinfo_disable_gpgconf ();
       return 0;
     }
+  else if (!strcmp (name, "require-gnupg"))
+    return _gpgme_set_engine_minimal_version (value);
   else if (!strcmp (name, "gpgconf-name"))
     return _gpgme_set_default_gpgconf_name (value);
   else if (!strcmp (name, "gpg-name"))
     return _gpgme_set_default_gpg_name (value);
+  else if (!strcmp (name, "inst-type"))
+    {
+      _gpgme_set_get_inst_type (value);
+      return 0;
+    }
+  else if (!strcmp (name, "w32-inst-dir"))
+    return _gpgme_set_override_inst_dir (value);
   else
     return -1;
 }
@@ -88,10 +99,10 @@ gpgme_new (gpgme_ctx_t *r_ctx)
 {
   gpgme_error_t err;
   gpgme_ctx_t ctx;
-  TRACE_BEG (DEBUG_CTX, "gpgme_new", r_ctx);
+  TRACE_BEG (DEBUG_CTX, "gpgme_new", r_ctx, "");
 
   if (_gpgme_selftest)
-    return TRACE_ERR (gpgme_error (_gpgme_selftest));
+    return TRACE_ERR (_gpgme_selftest);
 
   if (!r_ctx)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
@@ -153,7 +164,8 @@ gpgme_new (gpgme_ctx_t *r_ctx)
 
   *r_ctx = ctx;
 
-  return TRACE_SUC1 ("ctx=%p", ctx);
+  TRACE_SUC ("ctx=%p", ctx);
+  return 0;
 }
 
 
@@ -164,7 +176,7 @@ _gpgme_cancel_with_err (gpgme_ctx_t ctx, gpg_error_t ctx_err,
   gpgme_error_t err;
   struct gpgme_io_event_done_data data;
 
-  TRACE_BEG2 (DEBUG_CTX, "_gpgme_cancel_with_err", ctx, "ctx_err=%i, op_err=%i",
+  TRACE_BEG  (DEBUG_CTX, "_gpgme_cancel_with_err", ctx, "ctx_err=%i, op_err=%i",
 	      ctx_err, op_err);
 
   if (ctx_err)
@@ -195,7 +207,7 @@ gpgme_cancel (gpgme_ctx_t ctx)
 {
   gpg_error_t err;
 
-  TRACE_BEG (DEBUG_CTX, "gpgme_cancel", ctx);
+  TRACE_BEG (DEBUG_CTX, "gpgme_cancel", ctx, "");
 
   if (!ctx)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
@@ -210,7 +222,7 @@ gpgme_cancel (gpgme_ctx_t ctx)
 gpgme_error_t
 gpgme_cancel_async (gpgme_ctx_t ctx)
 {
-  TRACE_BEG (DEBUG_CTX, "gpgme_cancel_async", ctx);
+  TRACE_BEG (DEBUG_CTX, "gpgme_cancel_async", ctx, "");
 
   if (!ctx)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
@@ -227,7 +239,7 @@ gpgme_cancel_async (gpgme_ctx_t ctx)
 void
 gpgme_release (gpgme_ctx_t ctx)
 {
-  TRACE (DEBUG_CTX, "gpgme_release", ctx);
+  TRACE (DEBUG_CTX, "gpgme_release", ctx, "");
 
   if (!ctx)
     return;
@@ -238,12 +250,17 @@ gpgme_release (gpgme_ctx_t ctx)
   _gpgme_release_result (ctx);
   _gpgme_signers_clear (ctx);
   _gpgme_sig_notation_clear (ctx);
-  if (ctx->signers)
-    free (ctx->signers);
-  if (ctx->lc_ctype)
-    free (ctx->lc_ctype);
-  if (ctx->lc_messages)
-    free (ctx->lc_messages);
+  free (ctx->sender);
+  free (ctx->signers);
+  free (ctx->lc_ctype);
+  free (ctx->lc_messages);
+  free (ctx->override_session_key);
+  free (ctx->request_origin);
+  free (ctx->auto_key_locate);
+  free (ctx->trust_model);
+  free (ctx->cert_expire);
+  free (ctx->key_origin);
+  free (ctx->import_filter);
   _gpgme_engine_info_release (ctx->engine_info);
   ctx->engine_info = NULL;
   DESTROY_LOCK (ctx->lock);
@@ -311,10 +328,12 @@ _gpgme_release_result (gpgme_ctx_t ctx)
 }
 
 
+/* Note that setting the protocol will intentionally not fail if the
+ * engine is not available.  */
 gpgme_error_t
 gpgme_set_protocol (gpgme_ctx_t ctx, gpgme_protocol_t protocol)
 {
-  TRACE_BEG2 (DEBUG_CTX, "gpgme_set_protocol", ctx, "protocol=%i (%s)",
+  TRACE_BEG  (DEBUG_CTX, "gpgme_set_protocol", ctx, "protocol=%i (%s)",
 	      protocol, gpgme_get_protocol_name (protocol)
 	      ? gpgme_get_protocol_name (protocol) : "invalid");
 
@@ -335,7 +354,7 @@ gpgme_set_protocol (gpgme_ctx_t ctx, gpgme_protocol_t protocol)
       /* Shut down the engine when switching protocols.  */
       if (ctx->engine)
 	{
-	  TRACE_LOG1 ("releasing ctx->engine=%p", ctx->engine);
+	  TRACE_LOG  ("releasing ctx->engine=%p", ctx->engine);
 	  _gpgme_engine_release (ctx->engine);
 	  ctx->engine = NULL;
 	}
@@ -349,7 +368,7 @@ gpgme_set_protocol (gpgme_ctx_t ctx, gpgme_protocol_t protocol)
 gpgme_protocol_t
 gpgme_get_protocol (gpgme_ctx_t ctx)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_get_protocol", ctx,
+  TRACE (DEBUG_CTX, "gpgme_get_protocol", ctx,
 	  "ctx->protocol=%i (%s)", ctx->protocol,
 	  gpgme_get_protocol_name (ctx->protocol)
 	  ? gpgme_get_protocol_name (ctx->protocol) : "invalid");
@@ -361,7 +380,7 @@ gpgme_get_protocol (gpgme_ctx_t ctx)
 gpgme_error_t
 gpgme_set_sub_protocol (gpgme_ctx_t ctx, gpgme_protocol_t protocol)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_set_sub_protocol", ctx, "protocol=%i (%s)",
+  TRACE (DEBUG_CTX, "gpgme_set_sub_protocol", ctx, "protocol=%i (%s)",
 	  protocol, gpgme_get_protocol_name (protocol)
 	  ? gpgme_get_protocol_name (protocol) : "invalid");
 
@@ -376,7 +395,7 @@ gpgme_set_sub_protocol (gpgme_ctx_t ctx, gpgme_protocol_t protocol)
 gpgme_protocol_t
 gpgme_get_sub_protocol (gpgme_ctx_t ctx)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_get_sub_protocol", ctx,
+  TRACE (DEBUG_CTX, "gpgme_get_sub_protocol", ctx,
 	  "ctx->sub_protocol=%i (%s)", ctx->sub_protocol,
 	  gpgme_get_protocol_name (ctx->sub_protocol)
 	  ? gpgme_get_protocol_name (ctx->sub_protocol) : "invalid");
@@ -422,17 +441,53 @@ gpgme_get_protocol_name (gpgme_protocol_t protocol)
     }
 }
 
+
+/* Store the sender's address in the context.  ADDRESS is addr-spec of
+ * mailbox but my also be a complete mailbox, in which case this
+ * function extracts the addr-spec from it.  Returns 0 on success or
+ * an error code if no valid addr-spec could be extracted from
+ * ADDRESS.  */
+gpgme_error_t
+gpgme_set_sender (gpgme_ctx_t ctx, const char *address)
+{
+  char *p = NULL;
+
+  TRACE_BEG  (DEBUG_CTX, "gpgme_set_sender", ctx, "sender='%s'",
+              address?address:"(null)");
+
+  if (!ctx || (address && !(p = _gpgme_mailbox_from_userid (address))))
+    return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
+
+  free (ctx->sender);
+  ctx->sender = p;
+  return TRACE_ERR (0);
+}
+
+
+/* Return the sender's address (addr-spec part) from the context or
+ * NULL if none was set.  The returned value is valid as long as the
+ * CTX is valid and gpgme_set_sender has not been used.  */
+const char *
+gpgme_get_sender (gpgme_ctx_t ctx)
+{
+  TRACE (DEBUG_CTX, "gpgme_get_sender", ctx, "sender='%s'",
+          ctx?ctx->sender:"");
+
+  return ctx->sender;
+}
+
+
 /* Enable or disable the use of an ascii armor for all output.  */
 void
 gpgme_set_armor (gpgme_ctx_t ctx, int use_armor)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_set_armor", ctx, "use_armor=%i (%s)",
+  TRACE (DEBUG_CTX, "gpgme_set_armor", ctx, "use_armor=%i (%s)",
 	  use_armor, use_armor ? "yes" : "no");
 
   if (!ctx)
     return;
 
-  ctx->use_armor = use_armor;
+  ctx->use_armor = !!use_armor;
 }
 
 
@@ -440,12 +495,212 @@ gpgme_set_armor (gpgme_ctx_t ctx, int use_armor)
 int
 gpgme_get_armor (gpgme_ctx_t ctx)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_get_armor", ctx, "ctx->use_armor=%i (%s)",
+  TRACE (DEBUG_CTX, "gpgme_get_armor", ctx, "ctx->use_armor=%i (%s)",
 	  ctx->use_armor, ctx->use_armor ? "yes" : "no");
   return ctx->use_armor;
 }
 
 
+/* Set the flag NAME for CTX to VALUE.  Please consult the manual for
+ * a description of the flags.
+ */
+gpgme_error_t
+gpgme_set_ctx_flag (gpgme_ctx_t ctx, const char *name, const char *value)
+{
+  gpgme_error_t err = 0;
+  int abool;
+
+  TRACE (DEBUG_CTX, "gpgme_set_ctx_flag", ctx,
+          "name='%s' value='%s'",
+	  name? name:"(null)", value?value:"(null)");
+
+  abool = (value && *value)? !!atoi (value) : 0;
+
+  if (!ctx || !name || !value)
+    err = gpg_error (GPG_ERR_INV_VALUE);
+  else if (!strcmp (name, "redraw"))
+    {
+      ctx->redraw_suggested = abool;
+    }
+  else if (!strcmp (name, "full-status"))
+    {
+      ctx->full_status = abool;
+    }
+  else if (!strcmp (name, "raw-description"))
+    {
+      ctx->raw_description = abool;
+    }
+  else if (!strcmp (name, "export-session-key"))
+    {
+      ctx->export_session_keys = abool;
+    }
+  else if (!strcmp (name, "override-session-key"))
+    {
+      free (ctx->override_session_key);
+      ctx->override_session_key = strdup (value);
+      if (!ctx->override_session_key)
+        err = gpg_error_from_syserror ();
+    }
+  else if (!strcmp (name, "include-key-block"))
+    {
+      ctx->include_key_block = abool;
+    }
+  else if (!strcmp (name, "auto-key-import"))
+    {
+      ctx->auto_key_import = abool;
+    }
+  else if (!strcmp (name, "auto-key-retrieve"))
+    {
+      ctx->auto_key_retrieve = abool;
+    }
+  else if (!strcmp (name, "request-origin"))
+    {
+      free (ctx->request_origin);
+      ctx->request_origin = strdup (value);
+      if (!ctx->request_origin)
+        err = gpg_error_from_syserror ();
+    }
+  else if (!strcmp (name, "no-symkey-cache"))
+    {
+      ctx->no_symkey_cache = abool;
+    }
+  else if (!strcmp (name, "ignore-mdc-error"))
+    {
+      ctx->ignore_mdc_error = abool;
+    }
+  else if (!strcmp (name, "auto-key-locate"))
+    {
+      free (ctx->auto_key_locate);
+      ctx->auto_key_locate = strdup (value);
+      if (!ctx->auto_key_locate)
+        err = gpg_error_from_syserror ();
+    }
+  else if (!strcmp (name, "trust-model"))
+    {
+      free (ctx->trust_model);
+      ctx->trust_model = strdup (value);
+      if (!ctx->trust_model)
+        err = gpg_error_from_syserror ();
+    }
+  else if (!strcmp (name, "extended-edit"))
+    {
+      ctx->extended_edit = abool;
+    }
+  else if (!strcmp (name, "cert-expire"))
+    {
+      free (ctx->cert_expire);
+      ctx->cert_expire = strdup (value);
+      if (!ctx->cert_expire)
+        err = gpg_error_from_syserror ();
+    }
+  else if (!strcmp (name, "key-origin"))
+    {
+      free (ctx->key_origin);
+      ctx->key_origin = strdup (value);
+      if (!ctx->key_origin)
+        err = gpg_error_from_syserror ();
+    }
+  else if (!strcmp (name, "import-filter"))
+    {
+      free (ctx->import_filter);
+      ctx->import_filter = strdup (value);
+      if (!ctx->import_filter)
+        err = gpg_error_from_syserror ();
+    }
+  else if (!strcmp (name, "no-auto-check-trustdb"))
+    {
+      ctx->no_auto_check_trustdb = abool;
+    }
+  else
+    err = gpg_error (GPG_ERR_UNKNOWN_NAME);
+
+  return err;
+}
+
+
+/* Get the context flag named NAME.  See gpgme_set_ctx_flag for a list
+ * of valid names.  If the NAME is unknown NULL is returned.  For a
+ * boolean flag an empty string is returned for False and the string
+ * "1" for True; thus either atoi or a simple string test can be
+ * used.  */
+const char *
+gpgme_get_ctx_flag (gpgme_ctx_t ctx, const char *name)
+{
+  if (!ctx || !name)
+    return NULL;
+  else if (!strcmp (name, "redraw"))
+    {
+      return ctx->redraw_suggested? "1":"";
+    }
+  else if (!strcmp (name, "full-status"))
+    {
+      return ctx->full_status? "1":"";
+    }
+  else if (!strcmp (name, "raw-description"))
+    {
+      return ctx->raw_description? "1":"";
+    }
+  else if (!strcmp (name, "export-session-key"))
+    {
+      return ctx->export_session_keys? "1":"";
+    }
+  else if (!strcmp (name, "override-session-key"))
+    {
+      return ctx->override_session_key? ctx->override_session_key : "";
+    }
+  else if (!strcmp (name, "include-key-block"))
+    {
+      return ctx->include_key_block? "1":"";
+    }
+  else if (!strcmp (name, "auto-key-import"))
+    {
+      return ctx->auto_key_import? "1":"";
+    }
+  else if (!strcmp (name, "auto-key-retrieve"))
+    {
+      return ctx->auto_key_retrieve? "1":"";
+    }
+  else if (!strcmp (name, "request-origin"))
+    {
+      return ctx->request_origin? ctx->request_origin : "";
+    }
+  else if (!strcmp (name, "no-symkey-cache"))
+    {
+      return ctx->no_symkey_cache? "1":"";
+    }
+  else if (!strcmp (name, "ignore-mdc-error"))
+    {
+      return ctx->ignore_mdc_error? "1":"";
+    }
+  else if (!strcmp (name, "auto-key-locate"))
+    {
+      return ctx->auto_key_locate? ctx->auto_key_locate : "";
+    }
+  else if (!strcmp (name, "extended-edit"))
+    {
+      return ctx->extended_edit ? "1":"";
+    }
+  else if (!strcmp (name, "cert-expire"))
+    {
+      return ctx->cert_expire? ctx->cert_expire : "";
+    }
+  else if (!strcmp (name, "key-origin"))
+    {
+      return ctx->key_origin? ctx->key_origin : "";
+    }
+  else if (!strcmp (name, "import-filter"))
+    {
+      return ctx->import_filter? ctx->import_filter : "";
+    }
+  else if (!strcmp (name, "no-auto-check-trustdb"))
+    {
+      return ctx->no_auto_check_trustdb? "1":"";
+    }
+  else
+    return NULL;
+}
+
+
 /* Enable or disable the use of the special textmode.  Textmode is for
   example used for the RFC2015 signatures; note that the updated RFC
   3156 mandates that the MUA does some preparations so that textmode
@@ -453,20 +708,20 @@ gpgme_get_armor (gpgme_ctx_t ctx)
 void
 gpgme_set_textmode (gpgme_ctx_t ctx, int use_textmode)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_set_textmode", ctx, "use_textmode=%i (%s)",
+  TRACE (DEBUG_CTX, "gpgme_set_textmode", ctx, "use_textmode=%i (%s)",
 	  use_textmode, use_textmode ? "yes" : "no");
 
   if (!ctx)
     return;
 
-  ctx->use_textmode = use_textmode;
+  ctx->use_textmode = !!use_textmode;
 }
 
 /* Return the state of the textmode flag.  */
 int
 gpgme_get_textmode (gpgme_ctx_t ctx)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_get_textmode", ctx, "ctx->use_textmode=%i (%s)",
+  TRACE (DEBUG_CTX, "gpgme_get_textmode", ctx, "ctx->use_textmode=%i (%s)",
 	  ctx->use_textmode, ctx->use_textmode ? "yes" : "no");
   return ctx->use_textmode;
 }
@@ -477,20 +732,20 @@ gpgme_get_textmode (gpgme_ctx_t ctx)
 void
 gpgme_set_offline (gpgme_ctx_t ctx, int offline)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_set_offline", ctx, "offline=%i (%s)",
+  TRACE (DEBUG_CTX, "gpgme_set_offline", ctx, "offline=%i (%s)",
           offline, offline ? "yes" : "no");
 
   if (!ctx)
     return;
 
-  ctx->offline = offline;
+  ctx->offline = !!offline;
 }
 
 /* Return the state of the offline flag.  */
 int
 gpgme_get_offline (gpgme_ctx_t ctx)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_get_offline", ctx, "ctx->offline=%i (%s)",
+  TRACE (DEBUG_CTX, "gpgme_get_offline", ctx, "ctx->offline=%i (%s)",
           ctx->offline, ctx->offline ? "yes" : "no");
   return ctx->offline;
 }
@@ -512,7 +767,7 @@ gpgme_set_include_certs (gpgme_ctx_t ctx, int nr_of_certs)
   else
     ctx->include_certs = nr_of_certs;
 
-  TRACE2 (DEBUG_CTX, "gpgme_set_include_certs", ctx, "nr_of_certs=%i%s",
+  TRACE (DEBUG_CTX, "gpgme_set_include_certs", ctx, "nr_of_certs=%i%s",
 	  nr_of_certs, nr_of_certs == ctx->include_certs ? "" : " (-2)");
 }
 
@@ -522,7 +777,7 @@ gpgme_set_include_certs (gpgme_ctx_t ctx, int nr_of_certs)
 int
 gpgme_get_include_certs (gpgme_ctx_t ctx)
 {
-  TRACE1 (DEBUG_CTX, "gpgme_get_include_certs", ctx, "ctx->include_certs=%i",
+  TRACE (DEBUG_CTX, "gpgme_get_include_certs", ctx, "ctx->include_certs=%i",
 	  ctx->include_certs);
   return ctx->include_certs;
 }
@@ -534,12 +789,16 @@ gpgme_get_include_certs (gpgme_ctx_t ctx)
 gpgme_error_t
 gpgme_set_keylist_mode (gpgme_ctx_t ctx, gpgme_keylist_mode_t mode)
 {
-  TRACE1 (DEBUG_CTX, "gpgme_set_keylist_mode", ctx, "keylist_mode=0x%x",
+  TRACE (DEBUG_CTX, "gpgme_set_keylist_mode", ctx, "keylist_mode=0x%x",
 	  mode);
 
   if (!ctx)
     return gpg_error (GPG_ERR_INV_VALUE);
 
+  if ((mode & GPGME_KEYLIST_MODE_LOCATE_EXTERNAL) ==
+      (GPGME_KEYLIST_MODE_LOCAL|GPGME_KEYLIST_MODE_FORCE_EXTERN))
+    return gpg_error (GPG_ERR_INV_VALUE);
+
   ctx->keylist_mode = mode;
   return 0;
 }
@@ -549,7 +808,7 @@ gpgme_set_keylist_mode (gpgme_ctx_t ctx, gpgme_keylist_mode_t mode)
 gpgme_keylist_mode_t
 gpgme_get_keylist_mode (gpgme_ctx_t ctx)
 {
-  TRACE1 (DEBUG_CTX, "gpgme_get_keylist_mode", ctx,
+  TRACE (DEBUG_CTX, "gpgme_get_keylist_mode", ctx,
 	  "ctx->keylist_mode=0x%x", ctx->keylist_mode);
   return ctx->keylist_mode;
 }
@@ -559,7 +818,7 @@ gpgme_get_keylist_mode (gpgme_ctx_t ctx)
 gpgme_error_t
 gpgme_set_pinentry_mode (gpgme_ctx_t ctx, gpgme_pinentry_mode_t mode)
 {
-  TRACE1 (DEBUG_CTX, "gpgme_set_pinentry_mode", ctx, "pinentry_mode=%u",
+  TRACE (DEBUG_CTX, "gpgme_set_pinentry_mode", ctx, "pinentry_mode=%u",
 	  (unsigned int)mode);
 
   if (!ctx)
@@ -586,7 +845,7 @@ gpgme_set_pinentry_mode (gpgme_ctx_t ctx, gpgme_pinentry_mode_t mode)
 gpgme_pinentry_mode_t
 gpgme_get_pinentry_mode (gpgme_ctx_t ctx)
 {
-  TRACE1 (DEBUG_CTX, "gpgme_get_pinentry_mode", ctx,
+  TRACE (DEBUG_CTX, "gpgme_get_pinentry_mode", ctx,
 	  "ctx->pinentry_mode=%u", (unsigned int)ctx->pinentry_mode);
   return ctx->pinentry_mode;
 }
@@ -598,7 +857,7 @@ void
 gpgme_set_passphrase_cb (gpgme_ctx_t ctx, gpgme_passphrase_cb_t cb,
 			 void *cb_value)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_set_passphrase_cb", ctx,
+  TRACE (DEBUG_CTX, "gpgme_set_passphrase_cb", ctx,
 	  "passphrase_cb=%p/%p", cb, cb_value);
 
   if (!ctx)
@@ -615,7 +874,7 @@ void
 gpgme_get_passphrase_cb (gpgme_ctx_t ctx, gpgme_passphrase_cb_t *r_cb,
 			 void **r_cb_value)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_get_passphrase_cb", ctx,
+  TRACE (DEBUG_CTX, "gpgme_get_passphrase_cb", ctx,
 	  "ctx->passphrase_cb=%p/%p",
 	  ctx->passphrase_cb, ctx->passphrase_cb_value);
   if (r_cb)
@@ -630,7 +889,7 @@ gpgme_get_passphrase_cb (gpgme_ctx_t ctx, gpgme_passphrase_cb_t *r_cb,
 void
 gpgme_set_progress_cb (gpgme_ctx_t ctx, gpgme_progress_cb_t cb, void *cb_value)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_set_progress_cb", ctx, "progress_cb=%p/%p",
+  TRACE (DEBUG_CTX, "gpgme_set_progress_cb", ctx, "progress_cb=%p/%p",
 	  cb, cb_value);
 
   if (!ctx)
@@ -647,7 +906,7 @@ void
 gpgme_get_progress_cb (gpgme_ctx_t ctx, gpgme_progress_cb_t *r_cb,
 		       void **r_cb_value)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_get_progress_cb", ctx, "ctx->progress_cb=%p/%p",
+  TRACE (DEBUG_CTX, "gpgme_get_progress_cb", ctx, "ctx->progress_cb=%p/%p",
 	  ctx->progress_cb, ctx->progress_cb_value);
   if (r_cb)
     *r_cb = ctx->progress_cb;
@@ -661,7 +920,7 @@ gpgme_get_progress_cb (gpgme_ctx_t ctx, gpgme_progress_cb_t *r_cb,
 void
 gpgme_set_status_cb (gpgme_ctx_t ctx, gpgme_status_cb_t cb, void *cb_value)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_set_status_cb", ctx, "status_cb=%p/%p",
+  TRACE (DEBUG_CTX, "gpgme_set_status_cb", ctx, "status_cb=%p/%p",
 	  cb, cb_value);
 
   if (!ctx)
@@ -678,7 +937,7 @@ void
 gpgme_get_status_cb (gpgme_ctx_t ctx, gpgme_status_cb_t *r_cb,
 		       void **r_cb_value)
 {
-  TRACE2 (DEBUG_CTX, "gpgme_get_status_cb", ctx, "ctx->status_cb=%p/%p",
+  TRACE (DEBUG_CTX, "gpgme_get_status_cb", ctx, "ctx->status_cb=%p/%p",
 	  ctx ? ctx->status_cb : NULL, ctx ? ctx->status_cb_value : NULL);
 
   if (r_cb)
@@ -706,7 +965,7 @@ gpgme_set_io_cbs (gpgme_ctx_t ctx, gpgme_io_cbs_t io_cbs)
 
   if (io_cbs)
     {
-      TRACE6 (DEBUG_CTX, "gpgme_set_io_cbs", ctx,
+      TRACE (DEBUG_CTX, "gpgme_set_io_cbs", ctx,
 	      "io_cbs=%p (add=%p/%p, remove=%p, event=%p/%p",
 	      io_cbs, io_cbs->add, io_cbs->add_priv, io_cbs->remove,
 	      io_cbs->event, io_cbs->event_priv);
@@ -714,7 +973,7 @@ gpgme_set_io_cbs (gpgme_ctx_t ctx, gpgme_io_cbs_t io_cbs)
     }
   else
     {
-      TRACE1 (DEBUG_CTX, "gpgme_set_io_cbs", ctx,
+      TRACE (DEBUG_CTX, "gpgme_set_io_cbs", ctx,
 	      "io_cbs=%p (default)", io_cbs);
       ctx->io_cbs.add = NULL;
       ctx->io_cbs.add_priv = NULL;
@@ -731,8 +990,8 @@ gpgme_ssize_t
 gpgme_io_read (int fd, void *buffer, size_t count)
 {
   int ret;
-  TRACE_BEG2 (DEBUG_GLOBAL, "gpgme_io_read", fd,
-	      "buffer=%p, count=%u", buffer, count);
+  TRACE_BEG  (DEBUG_GLOBAL, "gpgme_io_read", fd,
+	      "buffer=%p, count=%zu", buffer, count);
 
   ret = _gpgme_io_read (fd, buffer, count);
 
@@ -747,8 +1006,8 @@ gpgme_ssize_t
 gpgme_io_write (int fd, const void *buffer, size_t count)
 {
   int ret;
-  TRACE_BEG2 (DEBUG_GLOBAL, "gpgme_io_write", fd,
-	      "buffer=%p, count=%u", buffer, count);
+  TRACE_BEG  (DEBUG_GLOBAL, "gpgme_io_write", fd,
+	      "buffer=%p, count=%zu", buffer, count);
 
   ret = _gpgme_io_write (fd, buffer, count);
 
@@ -766,8 +1025,8 @@ gpgme_io_writen (int fd, const void *buffer_arg, size_t count)
 {
   const char *buffer = buffer_arg;
   int ret = 0;
-  TRACE_BEG2 (DEBUG_GLOBAL, "gpgme_io_writen", fd,
-	      "buffer=%p, count=%u", buffer, count);
+  TRACE_BEG  (DEBUG_GLOBAL, "gpgme_io_writen", fd,
+	      "buffer=%p, count=%zu", buffer, count);
   while (count)
     {
       ret = _gpgme_io_write (fd, buffer, count);
@@ -785,7 +1044,7 @@ gpgme_io_writen (int fd, const void *buffer_arg, size_t count)
 void
 gpgme_get_io_cbs (gpgme_ctx_t ctx, gpgme_io_cbs_t io_cbs)
 {
-  TRACE6 (DEBUG_CTX, "gpgme_get_io_cbs", ctx,
+  TRACE (DEBUG_CTX, "gpgme_get_io_cbs", ctx,
 	  "io_cbs=%p, ctx->io_cbs.add=%p/%p, .remove=%p, .event=%p/%p",
 	  io_cbs, io_cbs->add, io_cbs->add_priv, io_cbs->remove,
 	  io_cbs->event, io_cbs->event_priv);
@@ -803,7 +1062,7 @@ gpgme_set_locale (gpgme_ctx_t ctx, int category, const char *value)
   char *new_lc_ctype = NULL;
   char *new_lc_messages = NULL;
 
-  TRACE_BEG2 (DEBUG_CTX, "gpgme_set_locale", ctx,
+  TRACE_BEG  (DEBUG_CTX, "gpgme_set_locale", ctx,
 	       "category=%i, value=%s", category, value ? value : "(null)");
 
 #define PREPARE_ONE_LOCALE(lcat, ucat)				\
@@ -872,7 +1131,7 @@ gpgme_set_locale (gpgme_ctx_t ctx, int category, const char *value)
 gpgme_engine_info_t
 gpgme_ctx_get_engine_info (gpgme_ctx_t ctx)
 {
-  TRACE1 (DEBUG_CTX, "gpgme_ctx_get_engine_info", ctx,
+  TRACE (DEBUG_CTX, "gpgme_ctx_get_engine_info", ctx,
 	  "ctx->engine_info=%p", ctx->engine_info);
   return ctx->engine_info;
 }
@@ -885,7 +1144,7 @@ gpgme_ctx_set_engine_info (gpgme_ctx_t ctx, gpgme_protocol_t proto,
 			   const char *file_name, const char *home_dir)
 {
   gpgme_error_t err;
-  TRACE_BEG4 (DEBUG_CTX, "gpgme_ctx_set_engine_info", ctx,
+  TRACE_BEG  (DEBUG_CTX, "gpgme_ctx_set_engine_info", ctx,
 	      "protocol=%i (%s), file_name=%s, home_dir=%s",
 	      proto, gpgme_get_protocol_name (proto)
 	      ? gpgme_get_protocol_name (proto) : "unknown",
@@ -898,7 +1157,7 @@ gpgme_ctx_set_engine_info (gpgme_ctx_t ctx, gpgme_protocol_t proto,
   /* Shut down the engine when changing engine info.  */
   if (ctx->engine)
     {
-      TRACE_LOG1 ("releasing ctx->engine=%p", ctx->engine);
+      TRACE_LOG  ("releasing ctx->engine=%p", ctx->engine);
       _gpgme_engine_release (ctx->engine);
       ctx->engine = NULL;
     }
@@ -930,7 +1189,7 @@ _gpgme_sig_notation_clear (gpgme_ctx_t ctx)
 void
 gpgme_sig_notation_clear (gpgme_ctx_t ctx)
 {
-  TRACE (DEBUG_CTX, "gpgme_sig_notation_clear", ctx);
+  TRACE (DEBUG_CTX, "gpgme_sig_notation_clear", ctx, "");
 
   if (!ctx)
     return;
@@ -952,7 +1211,7 @@ gpgme_sig_notation_add (gpgme_ctx_t ctx, const char *name,
   gpgme_sig_notation_t notation;
   gpgme_sig_notation_t *lastp;
 
-  TRACE_BEG3 (DEBUG_CTX, "gpgme_sig_notation_add", ctx,
+  TRACE_BEG  (DEBUG_CTX, "gpgme_sig_notation_add", ctx,
 	      "name=%s, value=%s, flags=0x%x",
 	      name ? name : "(null)", value ? value : "(null)",
 	      flags);
@@ -985,50 +1244,79 @@ gpgme_sig_notation_get (gpgme_ctx_t ctx)
 {
   if (!ctx)
     {
-      TRACE (DEBUG_CTX, "gpgme_sig_notation_get", ctx);
+      TRACE (DEBUG_CTX, "gpgme_sig_notation_get", ctx, "");
       return NULL;
     }
-  TRACE1 (DEBUG_CTX, "gpgme_sig_notation_get", ctx,
+  TRACE (DEBUG_CTX, "gpgme_sig_notation_get", ctx,
 	  "ctx->sig_notations=%p", ctx->sig_notations);
 
   return ctx->sig_notations;
 }
 
+
 
+/* Return a public key algorithm string made of the algorithm and size
+   or the curve name.  May return NULL on error.  Caller must free the
+   result using gpgme_free.  */
+char *
+gpgme_pubkey_algo_string (gpgme_subkey_t subkey)
+{
+  const char *prefix = NULL;
+  char *result;
+
+  if (!subkey)
+    {
+      gpg_err_set_errno (EINVAL);
+      return NULL;
+    }
+
+  switch (subkey->pubkey_algo)
+    {
+    case GPGME_PK_RSA:
+    case GPGME_PK_RSA_E:
+    case GPGME_PK_RSA_S: prefix = "rsa"; break;
+    case GPGME_PK_ELG_E: prefix = "elg"; break;
+    case GPGME_PK_DSA:	 prefix = "dsa"; break;
+    case GPGME_PK_ELG:   prefix = "xxx"; break;
+    case GPGME_PK_ECC:
+    case GPGME_PK_ECDH:
+    case GPGME_PK_ECDSA:
+    case GPGME_PK_EDDSA: prefix = "";    break;
+    }
+
+  if (prefix && *prefix)
+    {
+      char buffer[40];
+      snprintf (buffer, sizeof buffer, "%s%u", prefix, subkey->length);
+      result = strdup (buffer);
+    }
+  else if (prefix && subkey->curve && *subkey->curve)
+    result = strdup (subkey->curve);
+  else if (prefix)
+    result =  strdup ("E_error");
+  else
+    result = strdup  ("unknown");
+
+  return result;
+}
+
+
 const char *
 gpgme_pubkey_algo_name (gpgme_pubkey_algo_t algo)
 {
   switch (algo)
     {
-    case GPGME_PK_RSA:
-      return "RSA";
-
-    case GPGME_PK_RSA_E:
-      return "RSA-E";
-
-    case GPGME_PK_RSA_S:
-      return "RSA-S";
-
-    case GPGME_PK_ELG_E:
-      return "ELG-E";
-
-    case GPGME_PK_DSA:
-      return "DSA";
-
-    case GPGME_PK_ECC:
-      return "ECC";
-
-    case GPGME_PK_ELG:
-      return "ELG";
-
-    case GPGME_PK_ECDSA:
-      return "ECDSA";
-
-    case GPGME_PK_ECDH:
-      return "ECDH";
-
-    default:
-      return NULL;
+    case GPGME_PK_RSA:   return "RSA";
+    case GPGME_PK_RSA_E: return "RSA-E";
+    case GPGME_PK_RSA_S: return "RSA-S";
+    case GPGME_PK_ELG_E: return "ELG-E";
+    case GPGME_PK_DSA:   return "DSA";
+    case GPGME_PK_ECC:   return "ECC";
+    case GPGME_PK_ELG:   return "ELG";
+    case GPGME_PK_ECDSA: return "ECDSA";
+    case GPGME_PK_ECDH:  return "ECDH";
+    case GPGME_PK_EDDSA: return "EdDSA";
+    default:             return NULL;
     }
 }
 
